clear

%% Loading values
target_file=fopen('train-labels-idx1-ubyte','r');
fread(target_file,4); % magic number
n=sum(256.^(3:-1:0).*fread(target_file,4)'); % number of elements
target=fread(target_file)'; % saving targets
fclose(target_file);
target2=zeros(10,length(target));
for ii=1:length(target)
    target2(:,ii)=((0:9)==target(ii)); % Transforming digit into 10x1 vector
end

image_file=fopen('train-images-idx3-ubyte','r');
fread(image_file,4); % magic number
fread(image_file,4); % number of elements
row=sum(256.^(3:-1:0).*fread(image_file,4)'); % number of rows
col=sum(256.^(3:-1:0).*fread(image_file,4)'); % number of columns
images=zeros(row*col,n);
for ii=1:n
    images(:,ii)=fread(image_file,row*col);
end
fclose(image_file);

disp('Training data loaded')





%% Visualizing sample of data

k=randi([1 n], [4 4]); % random integers
target(k)'
figure(1)
for ii=1:16
    subplot(4,4,ii)
    imagesc(reshape(images(:,k(ii)),[row col])')
    axis square; axis off
end




%% Neural network Training

% Solve a Pattern Recognition Problem with a Neural Network
% Script generated by NPRTOOL

inputs = images;
targets = target2;

% Create a Pattern Recognition Network
hiddenLayerSize = 100;
net = patternnet(hiddenLayerSize);

% Set up Division of Data for Training, Validation, Testing
net.divideParam.trainRatio = 70/100;
net.divideParam.valRatio = 15/100;
net.divideParam.testRatio = 15/100;

% Train the Network
[net,tr] = train(net,inputs,targets);

% Test the Network
outputs = net(inputs);
[~, indx]=max(outputs);
errors = 100*(n-sum(target==indx-1))/n % Error in percent

% View the Network
view(net)

% Plots
% Uncomment these lines to enable various plots.
% figure, plotperform(tr)
% figure, plottrainstate(tr)
% figure, plotconfusion(targets,outputs)
% figure, ploterrhist(errors)





%% Checking on the previous sample
outputs = net(inputs(:,k));
[m, indx]=max(outputs);
reshape(indx-1,4,4)'
(16-sum(target(reshape(k,16,1))==indx-1)) % Number of errors






%% Testing on more data
target_file=fopen('t10k-labels.idx1-ubyte','r');
fread(target_file,4); % magic number
n_test=sum(256.^(3:-1:0).*fread(target_file,4)'); % number of elements
target_t=fread(target_file)'; % saving targets
fclose(target_file);
target_test=zeros(10,length(target_t));
for ii=1:length(target_t)
    target_test(:,ii)=((0:9)==target_t(ii));
end

image_file=fopen('t10k-images.idx3-ubyte','r');
fread(image_file,4); % magic number
fread(image_file,4); % number of elements
row=sum(256.^(3:-1:0).*fread(image_file,4)'); % number of elements
col=sum(256.^(3:-1:0).*fread(image_file,4)'); % number of elements
images_test=zeros(row*col,n_test);
for ii=1:n_test
    images_test(:,ii)=fread(image_file,row*col);
end
fclose(image_file);

outputs = net(images_test);
[~, indx]=max(outputs);
100*(n_test-sum(target_t==indx-1))/n_test % Error in percent






%% Test on handwritten digits
N=col;
A=zeros(N,N);
figure
imagesc(A); axis square ; axis off

[X, Y]=meshgrid(1:N);
B=exp(-((X-N/2).^2+(Y-N/2).^2)/(1.5)^2)/100; % This is a 2D gaussian used to spread the lines

while 1 % Infinite loop, close window to stop process
    
    hFH = imfreehand('Closed',false); % Ask for user to draw on the figure
    xy = hFH.getPosition; % Gets x, y position of the edges of the drawing
    
    xy=round(xy); % Rounds to integer
    xy=max(xy,ones(size(xy))); % Positions below one are set to one
    xy=min(xy,N*ones(size(xy))); % Positions above N become N
    
    z=xy(:,2)+(xy(:,1)-1)*N; % Transforms the xy coordinates into single index for image A
    
    A=zeros(N,N); % Image
    A(z)=1; % Sets all pixels drawn by user to 1
    A = conv2(A,B,'same'); % Convolutes with Gaussian to spread the line
    A=255*A./max(max(A)); % Renomalizes
    offX=round(N/2-sum(sum(X.*A))/sum(sum(A))); % Checks where center of gravity is from center
    offY=round(N/2-sum(sum(Y.*A))/sum(sum(A))); % Checks where center of gravity is from center
    A=circshift(A,[offY offX]); % shifts the image towards the center 
    
    imagesc(A); axis square ; axis off
    resp=net(reshape(A',[N*N 1])); % Puts the image through the network
    [m, indx]=max(resp); % Gets result
    title(strcat('This number is a',int2str((indx-1)))) % Shows the result in the title
end